@@ -24,7 +24,8 @@
 
 #include "MSData.hpp"
 #include "pwiz/utility/misc/Std.hpp"
-#include <boost/lexical_cast.hpp>
+#include <boost/range/adaptor/map.hpp>
+#include "pwiz/utility/misc/optimized_lexical_cast.hpp"
 #include "Diff.hpp"
 
 namespace pwiz {
@@ -555,6 +556,9 @@ PWIZ_API_DECL string translateScanNumberToNativeID(CVID nativeIdFormat, const st
         case MS_scan_number_only_nativeID_format:
             return "scan=" + scanNumber;
 
+        case MS_Bruker_TSF_nativeID_format:
+            return "frame=" + scanNumber;
+
         default:
             return "";
     }
@@ -585,6 +589,9 @@ PWIZ_API_DECL string translateNativeIDToScanNumber(CVID nativeIdFormat, const st
         case MS_scan_number_only_nativeID_format:
             return value(id, "scan");
 
+        case MS_Bruker_TSF_nativeID_format:
+            return value(id, "frame");
+
         default:
             if (bal::starts_with(id, "scan=")) return value(id, "scan");
             else if (bal::starts_with(id, "index=")) return value(id, "index");
@@ -635,6 +642,7 @@ PWIZ_API_DECL bool Spectrum::empty() const
            precursors.empty() && 
            products.empty() && 
            binaryDataArrayPtrs.empty() &&
+           integerDataArrayPtrs.empty() &&
            ParamContainer::empty();
 }
 
@@ -909,6 +917,7 @@ PWIZ_API_DECL bool Chromatogram::empty() const
            precursor.empty() &&
            product.empty() &&
            binaryDataArrayPtrs.empty() &&
+           integerDataArrayPtrs.empty() &&
            ParamContainer::empty();
 }
 
@@ -1112,6 +1121,8 @@ PWIZ_API_DECL size_t SpectrumList::findAbbreviated(const string& abbreviatedId,
         if ((result >= 0 && result < size()) || s == 0)
             return result;
     }
+
+    return size();
 }
 
 
@@ -1134,6 +1145,24 @@ PWIZ_API_DECL IndexList SpectrumList::findSpotID(const string& spotID) const
     return result;
 }
 
+/// returns true iff the id formats match
+PWIZ_API_DECL bool SpectrumList::checkNativeIdMatch(const string& firstIdInFile, const string& searchedId) const
+{
+    auto actualId = pwiz::msdata::id::parse(firstIdInFile);
+    auto actualIdKeys = actualId | boost::adaptors::map_keys;
+    auto actualIdKeySet = std::set<std::string>(actualIdKeys.begin(), actualIdKeys.end());
+
+    auto expectedId = pwiz::msdata::id::parse(searchedId);
+    auto expectedIdKeys = expectedId | boost::adaptors::map_keys;
+    auto expectedIdKeySet = std::set<std::string>(expectedIdKeys.begin(), expectedIdKeys.end());
+
+    std::vector<std::string> missingIdKeys;
+    std::set_symmetric_difference(expectedIdKeySet.begin(), expectedIdKeySet.end(),
+        actualIdKeySet.begin(), actualIdKeySet.end(),
+        std::back_inserter(missingIdKeys));
+
+    return missingIdKeys.empty();
+}
 
 PWIZ_API_DECL const shared_ptr<const DataProcessing> SpectrumList::dataProcessingPtr() const
 {
@@ -1156,11 +1185,6 @@ PWIZ_API_DECL SpectrumPtr SpectrumList::spectrum(size_t index, DetailLevel detai
     return spectrum(index, detailLevel == DetailLevel_FullData);
 }
 
-
-PWIZ_API_DECL void SpectrumList::warn_once(const char *msg) const
-{
-}
-
 PWIZ_API_DECL DetailLevel SpectrumList::min_level_accepted(std::function<boost::tribool(const Spectrum&)> predicate) const
 {
     DetailLevel result = DetailLevel_InstantMetadata;
@@ -1189,6 +1213,10 @@ PWIZ_API_DECL DetailLevel SpectrumList::min_level_accepted(std::function<boost::
     throw runtime_error("[SpectrumList::min_level_accepted] no spectrum satisfied the given predicate at any DetailLevel");
 }
 
+PWIZ_API_DECL bool SpectrumList::calibrationSpectraAreOmitted() const
+{
+    return false; // Default implementation, currently only Waters lockmass functions are actually handled
+}
 
 //
 // SpectrumListSimple
@@ -1363,6 +1391,25 @@ PWIZ_API_DECL vector<DataProcessingPtr> MSData::allDataProcessingPtrs() const
             result.push_back(boost::const_pointer_cast<DataProcessing>(sldp));
     }
 
+    /* CONSIDER: alternate merging approach
+    if (run.spectrumListPtr.get() && run.spectrumListPtr->dataProcessingPtr().get())
+    {
+        // if SpectrumList::dataProcessingPtr() is not in MSData::dataProcessingPtrs, add it
+        const shared_ptr<const DataProcessing> sldp = run.spectrumListPtr->dataProcessingPtr();
+        auto dpInMsdItr = std::find_if(result.begin(), result.end(), HasID<DataProcessing>(sldp->id));
+        if (dpInMsdItr != result.end())
+        {
+            Diff<DataProcessing, DiffConfig> diff(*sldp, **dpInMsdItr);
+            if (diff)
+            {
+                (*dpInMsdItr)->id += "_old";
+                result.push_back(boost::const_pointer_cast<DataProcessing>(sldp));
+            }
+        }
+        else
+            result.push_back(boost::const_pointer_cast<DataProcessing>(sldp));
+    }*/
+
     if (run.chromatogramListPtr.get())
     {
         // if ChromatogramList::dataProcessingPtr() is not in MSData::dataProcessingPtrs, add it
