@@ -24,19 +24,21 @@
 
 #include "SpectrumListBase.hpp"
 #include "pwiz/utility/misc/String.hpp"
+#include "pwiz/utility/misc/Stream.hpp"
 #include <boost/thread/lock_guard.hpp>
 #include <boost/thread/mutex.hpp>
+#include <boost/functional/hash.hpp>
 
 
 namespace {
     boost::mutex m;
 }
 
-PWIZ_API_DECL void pwiz::msdata::SpectrumListBase::warn_once(const char * msg) const
+PWIZ_API_DECL void pwiz::msdata::ListBase::warn_once(const char * msg) const
 {
     boost::lock_guard<boost::mutex> g(m);
     if (warn_msg_hashes_.insert(hash(msg)).second) // .second is true iff value is new
-        std::cerr << msg << std::endl;
+        cerr << msg << std::endl;
 }
 
 
@@ -65,24 +67,11 @@ PWIZ_API_DECL size_t pwiz::msdata::SpectrumListBase::checkNativeIdFindResult(siz
             boost::lock_guard<boost::mutex> g(m);
 
             // early exit if warning already issued, to avoid potentially doing these calculations for thousands of ids
-            if (!warn_msg_hashes_.insert(spectrum_id_mismatch_hash_).second)
+            if (!impl_.warn_msg_hashes().insert(spectrum_id_mismatch_hash_).second)
                 return size();
         }
 
-        auto actualId = pwiz::msdata::id::parse(firstId);
-        auto actualIdKeys = actualId | boost::adaptors::map_keys;
-        auto actualIdKeySet = std::set<std::string>(actualIdKeys.begin(), actualIdKeys.end());
-
-        auto expectedId = pwiz::msdata::id::parse(id);
-        auto expectedIdKeys = expectedId | boost::adaptors::map_keys;
-        auto expectedIdKeySet = std::set<std::string>(expectedIdKeys.begin(), expectedIdKeys.end());
-
-        std::vector<std::string> missingIdKeys;
-        std::set_symmetric_difference(expectedIdKeySet.begin(), expectedIdKeySet.end(),
-            actualIdKeySet.begin(), actualIdKeySet.end(),
-            std::back_inserter(missingIdKeys));
-
-        if (!missingIdKeys.empty())
+        if (!checkNativeIdMatch(firstId, id))
             warn_once(("[SpectrumList::find] mismatch between spectrum id format of the file (" + firstId + ") and the looked-up id (" + id + ")").c_str());
         return size();
     }
@@ -93,7 +82,7 @@ PWIZ_API_DECL size_t pwiz::msdata::SpectrumListBase::checkNativeIdFindResult(siz
     }
 }
 
-size_t pwiz::msdata::SpectrumListBase::hash(const char* msg) const
+size_t pwiz::msdata::ListBase::hash(const char* msg) const
 {
     return boost::hash_range(msg, msg + strlen(msg));
-}
\ No newline at end of file
+}
